'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.EventEmitter = exports.buildEventCenterByName = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _isFunction = require('lodash/isFunction');

var _isFunction2 = _interopRequireDefault(_isFunction);

var _isNumber = require('lodash/isNumber');

var _isNumber2 = _interopRequireDefault(_isNumber);

var _isObject = require('lodash/isObject');

var _isObject2 = _interopRequireDefault(_isObject);

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function each(obj, iterator) {
  var context = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

  if ((0, _isObject2.default)(obj) && !(0, _isArray2.default)(obj)) {
    for (var key in obj) {
      if (!obj.hasOwnProperty(key)) continue;
      if (iterator.call(context, obj[key], key, obj) === false) break;
    }
  } else if ((0, _isNumber2.default)(obj.length)) {
    for (var i = 0; i < obj.length; i++) {
      if (iterator.call(context, obj[i], i, obj) === false) break;
    }
  }
}

var EVENT_CENTER_TYPE = {
  page: 'PageEventCenter',
  app: 'AppEventCenter',
  global: 'GlobalEventCenter'
};

/**
 * Slice event string such as:
 * balabala
 * balabala.xxx
 * page.balabala
 * @param  {String} ev            event string
 * @return {Object} result        slice result, an object of event info
 * @return {string} result.type   event type
 * @return {string} result.name   true event
 * @return {string} result.value  raw event passed in
 * @return {string} result.center event center to register in
 */

// PageEventCenter.back
function buildEventCenterByName(event) {
  // Page.back
  event = buildEventByName(event);
  event.center = EVENT_CENTER_TYPE[event.type];
  return event;
}

/**
 * Check if there is an array for holding event options in the options pool object, create one if not.
 * @param  {Object} optionsPool Options pool object
 * @param  {String} ev          Event name
 * @return {Array}              Options array of event
 */
function checkOptions(optionsPool, ev) {
  return optionsPool[ev] = optionsPool[ev] || [];
}

/**
 * Delete event options at indexes.
 * @param  {Array} options      Options array
 * @param  {Array}  indexes     Array of indexes to be deleted
 */
function deleteOptionsAtIndexes() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var indexes = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

  // Make indexes decrease and collaborate with `splice` to delete correctely
  each(indexes.sort().reverse(), function (index) {
    options.splice(index, 1);
  });
}

/**
 * Create a TypeError of callback
 * @param  {String} ev Event name
 * @return {TypeError}
 */
function callbackTypeError(ev) {
  return new TypeError('Event callback for ' + ev + ' must be function.');
}

/**
 * buildEventObject string such as:
 * balabala
 * balabala.xxx
 * page.balabala
 * @param  {String} ev            event string
 * @return {Object} result        slice result, an object of event info
 * @return {string} result.type   event type
 * @return {string} result.name   true event
 * @return {string} result.value  raw event passed in
 * @return {string} result.center event center to register in
 */
function buildEventByName(eventName) {
  var reg = /^(page|app|global)\.(.+)$/i;

  /**
   * if none of formats below, prefix default type.
   * page.foo
   * app.foo.bar
   * global.foo.bar
   */
  if (!reg.test(eventName)) {
    eventName = 'page.' + eventName;
  }

  var match = eventName.match(reg);
  /**
   * lowercase to support different prefix like:
   * page.close
   * Page.close
   * Global.DidBecomeActive
   */
  var type = match[1].toLowerCase();
  var name = match[2];

  return {
    type: type,
    name: name,
    value: type + '.' + name
  };
}

var EventEmitter = function () {
  function EventEmitter() {
    _classCallCheck(this, EventEmitter);

    this.optionsPool = {
      // click: [{notify, once: false}],
      // touch: [{notify, once: true}],
    };
  }

  /**
   * Register event
   * @param  {String}   ev              Event name
   * @param  {Function} callback        Event callback when event is emitted
   * @param  {Object}   settings        Register settings
   * @param  {Boolean}  settings.once   Indicate if callback can only be emitted once
   * @return {Promise}
   */


  _createClass(EventEmitter, [{
    key: 'on',
    value: function on(ev, callback) {
      // on('page.back')
      var options = checkOptions(this.optionsPool, ev);
      if ((0, _isFunction2.default)(callback)) {
        options.push({ notify: callback });
        return Promise.resolve();
      } else {
        var error = callbackTypeError(ev);
        return Promise.reject(error);
      }
    }

    /**
     * Unregister event
     * @param  {String}   ev              Event name
     * @param  {Function} callback        Event callback to be unregistered
     * @return {Promise}
     */

  }, {
    key: 'off',
    value: function off(ev, callback) {
      var options = checkOptions(this.optionsPool, ev);

      // Unregister all callbacks if no callback argument.
      if (!callback) {
        this.optionsPool[ev] = [];
        return Promise.resolve();
      }

      if ((0, _isFunction2.default)(callback)) {
        var indexes = [];
        each(options, function (o, i) {
          if (o.notify !== callback) return;
          indexes.push(i);
        });
        deleteOptionsAtIndexes(options, indexes);
        return Promise.resolve();
      } else {
        var error = callbackTypeError(ev);
        return Promise.reject(error);
      }
    }

    /**
     * Register event only once, automaticlly unregistered after the event is emitted.
     * @param  {String}   ev              Event name
     * @param  {Function} callback        Event callback when event is emitted
     * @return {Promise}
     */

  }, {
    key: 'once',
    value: function once(eventName, callback) {
      return this.on(eventName, callback, { once: true });
    }

    /**
     * Emit event
     * @param  {String} ev   Event name
     * @param  {Any}    arg[0] 1st arg passed to notify callback.
     * @param  {Any}    arg[1] 2nd arg passed to notify callback.
     * @param  {Any}    ...
     * @param  {Any}    arg[n] nth arg passed to notify callback.
     * @return {Promise}
     */

  }, {
    key: 'emit',
    value: function emit(ev) {
      for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = arguments[_key];
      }

      var options = checkOptions(this.optionsPool, ev);
      var indexes = [];
      each(options, function (o, i) {
        (0, _isFunction2.default)(o.notify) && o.notify.apply(o, args);
        o.once && indexes.push(i);
      });
      // Delete once options
      deleteOptionsAtIndexes(options, indexes);
      return Promise.resolve();
    }
  }]);

  return EventEmitter;
}();

exports.buildEventCenterByName = buildEventCenterByName;
exports.EventEmitter = EventEmitter;
exports['default'] = module.exports;
exports.default = module.exports;